from config import Config
from flask import Flask
from flask_apscheduler import APScheduler
from flask_login import LoginManager
from flask_mail import Mail
from flask_migrate import Migrate
from flask_paranoid import Paranoid
from flask_socketio import SocketIO
from flask_sqlalchemy import SQLAlchemy
from flask_hashids import Hashids
import flask_assets


assets: flask_assets.Environment = flask_assets.Environment()
db: SQLAlchemy = SQLAlchemy()
hashids: Hashids = Hashids()
login: LoginManager = LoginManager()
login.login_view = 'auth.login'
login.login_message = 'Please log in to access this page.'
mail: Mail = Mail()
migrate: Migrate = Migrate()
paranoid: Paranoid = Paranoid()
paranoid.redirect_view = '/'
scheduler: APScheduler = APScheduler()  # TODO: Use this!
socketio: SocketIO = SocketIO()


def create_app(config: Config = Config) -> Flask:
    ''' Creates an initialized Flask (WSGI Application) object. '''
    app: Flask = Flask(__name__)
    app.config.from_object(config)

    assets.init_app(app)
    config.init_app(app)
    db.init_app(app)
    hashids.init_app(app)
    login.init_app(app)
    mail.init_app(app)
    migrate.init_app(app, db)
    paranoid.init_app(app)
    socketio.init_app(
        app,
        message_queue=app.config['NOPAQUE_SOCKETIO_MESSAGE_QUEUE_URI']
    )

    from .events import socketio as socketio_events
    from .events import sqlalchemy as sqlalchemy_events

    from .admin import bp as admin_blueprint
    app.register_blueprint(admin_blueprint, url_prefix='/admin')

    from .api import bp as api_blueprint
    app.register_blueprint(api_blueprint, url_prefix='/api')

    from .auth import bp as auth_blueprint
    app.register_blueprint(auth_blueprint, url_prefix='/auth')

    from .contribute import bp as contribute_blueprint
    app.register_blueprint(contribute_blueprint, url_prefix='/contribute')

    from .corpora import bp as corpora_blueprint
    app.register_blueprint(corpora_blueprint, url_prefix='/corpora')

    from .errors import bp as errors_blueprint
    app.register_blueprint(errors_blueprint)

    from .jobs import bp as jobs_blueprint
    app.register_blueprint(jobs_blueprint, url_prefix='/jobs')

    from .main import bp as main_blueprint
    app.register_blueprint(main_blueprint)

    from .services import bp as services_blueprint
    app.register_blueprint(services_blueprint, url_prefix='/services')

    from .settings import bp as settings_blueprint
    app.register_blueprint(settings_blueprint, url_prefix='/settings')

    return app
